package cn.iocoder.yudao.module.shipment.listener;


import cn.iocoder.yudao.framework.apollo.core.event.box.BoxCheckOrderApprovalEvent;
import cn.iocoder.yudao.framework.apollo.core.event.box.BoxOrderForceWarehouseCheckEvent;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxLoadInfoDO;
import cn.iocoder.yudao.module.shipment.enums.BoxApprovalTypeEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxLoadInfo.BoxLoadInfoService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.WAREHOUSE_IN_BOX_EXCEPTION;
import static cn.iocoder.yudao.module.order.enums.OrderApprovalTypeResultEnum.*;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 订单强制到仓业务执行
 *
 * @author zhengYi
 */
@Component("BoxOrderForceWarehouseCheckListener")
@AllArgsConstructor
@Slf4j
public class BoxOrderForceWarehouseCheckListener {

    private final BoxLoadInfoService boxLoadInfoService;
    private final BoxService boxService;


    /**
     * 订单强制到仓业务执行
     *
     * @param event 订单信息
     */
    @EventListener(BoxOrderForceWarehouseCheckEvent.class)
    public void orderForceWarehouseCheck(BoxOrderForceWarehouseCheckEvent event) {
        BoxDO boxDO = boxService.getOne(new LambdaQueryWrapper<BoxDO>().eq(BoxDO::getSelfNo, event.getSelfNo()));
        if (boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        // 跟新到仓时间到卸柜/到仓箱标数据表
        boxLoadInfoService.update(new LambdaUpdateWrapper<BoxLoadInfoDO>()
                .set(BoxLoadInfoDO::getUnloadTime, event.getUnloadTime())
                .eq(BoxLoadInfoDO::getOrderId, event.getOrderId()));
        if (event.getTransportId() == 1){
            // 海运
            boxLoadInfoService.updateOrderStatus(Arrays.asList(event.getOrderId()), event.getUnloadTime(), boxDO);
        }else {
            // 空运
            boxLoadInfoService.updateOrderStatusAir(Arrays.asList(event.getOrderId()), event.getUnloadTime(), boxDO);
        }
    }


}
